"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _createForOfIteratorHelper2 = _interopRequireDefault(require("@babel/runtime/helpers/createForOfIteratorHelper"));

var _terminalKit = _interopRequireDefault(require("terminal-kit"));

var _ = _interopRequireDefault(require("../"));

var term = _terminalKit["default"].terminal;

function describeExpression(output, expr) {
  output.put({
    markup: true,
    y: 1,
    x: 0
  }, "Count: ^_".concat(expr.count, "^:"));
  output.put({
    markup: true,
    y: 2,
    x: 0
  }, "Command: ^_".concat(expr.command, "^:"));
  var positionals = expr.getPositionals();

  if (positionals.length) {
    output.put({
      y: 4,
      x: 0
    }, "Positionals (including ambiguous flag-payloads):");

    var _iterator = (0, _createForOfIteratorHelper2["default"])(positionals.entries()),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var _step$value = (0, _slicedToArray2["default"])(_step.value, 2),
            idx = _step$value[0],
            positional = _step$value[1];

        output.put({
          markup: true,
          y: 5 + idx,
          x: 0
        }, "".concat(idx, ". \"").concat(positional, "\""));
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }

  var flagsStart = positionals.length ? 7 + positionals.length : 5; // TODO: Hmm, I wonder if I should add a more efficient ‘are there any flags at all’ test ...

  if (expr.flags.length) {
    output.put({
      y: flagsStart - 1,
      x: 0
    }, "Flags:");
    expr.forEachFlag(function (flag, payload, idx) {
      if (payload) output.put({
        markup: true,
        y: flagsStart + idx,
        x: 0
      }, "- ".concat(flag, ": \"").concat(payload, "\""));else output.put({
        markup: true,
        y: flagsStart + idx,
        x: 0
      }, "- ".concat(flag, " (no payload)"));
    });
  }
}

function describeCheckpoint(output, cp) {
  var state = cp.automaton_status;

  switch (state) {
    case 'Accepted':
    case 'Rejected':
      output.put({
        markup: true,
        x: 0,
        y: 0
      }, "State: ^_".concat(state, "^:"));
      break;

    case 'InputNeeded':
    case 'Shifting':
    case 'AboutToReduce':
    case 'HandlingError':
      var command = cp.command,
          symbol = cp.incoming_symbol,
          type = cp.incoming_symbol_type,
          category = cp.incoming_symbol_category;
      if (typeof symbol === 'undefined') output.put({
        markup: true,
        x: 0,
        y: 0
      }, "State: ^_".concat(state, "^: (initial)"));else {
        var symbol_desc = "".concat(symbol, " : (").concat(type, ") ").concat(category);
        output.put({
          markup: true,
          x: 0,
          y: 0
        }, "State: ^_".concat(state, "^:, incoming: ^_").concat(symbol_desc, "^:, current command: ^_").concat(command, "^:"));
      }
  }

  draw(output);
}

function displayStack(output, cp) {
  var _iterator2 = (0, _createForOfIteratorHelper2["default"])(cp.beforeStack),
      _step2;

  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var el = _step2.value;
      output.put({}, el.incoming_symbol);
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }
}

function onChange(input, output) {
  var textContent = input.getText();

  var start = _["default"].Parser.startExpressionWithString(textContent);

  output.fill({
    "char": ' '
  });
  output.moveTo(0, 0);
  start["continue"]({
    onAccept: function onAccept(expr) {
      output.put({}, 'Input accepted!\n');
      describeExpression(output, expr);
    },
    onFail: function onFail(lastGood, errorAt) {
      displayStack(output, lastGood);
    }
  });
  draw(output);
} // ### Ignore me, mundane terminal-setup noise follows.


function handleKeypress(buf, key) {
  var keyDesc = '^y ' + key + ' '; // In terminal-kit, `key` is always more than one codepoint in length if it's a special keypress.

  if (key.length === 1) {
    showNotice(keyDesc, false);
    buf.insert(key);
    buf.drawCursor();
    draw(buf, true);
  } else {
    switch (key) {
      // look ma, I'm implementing a text-editor in JavaScript. <.<
      case 'LEFT':
        showNotice(keyDesc, true);
        buf.moveBackward();
        buf.drawCursor();
        buf.dst.drawCursor();
        break;

      case 'RIGHT':
        showNotice(keyDesc, true);
        buf.moveForward();
        buf.drawCursor();
        buf.dst.drawCursor();
        break;

      case 'UP':
        showNotice(keyDesc, true);
        buf.moveUp();
        buf.moveInBound();
        buf.drawCursor();
        buf.dst.drawCursor();
        break;

      case 'DOWN':
        showNotice(keyDesc, true);
        buf.moveDown();
        buf.moveInBound();
        buf.drawCursor();
        buf.dst.drawCursor();
        break;

      case 'BACKSPACE':
        showNotice(keyDesc, false);
        buf.backDelete();
        buf.drawCursor();
        draw(buf, true);
        break;

      case 'ENTER':
        showNotice(keyDesc, false);
        buf.newLine();
        buf.drawCursor();
        draw(buf, true);
        break;

      case 'CTRL_C':
        showNotice(' Buh-bye 💖 ! ', true);
        term.processExit(1);
        break;

      default:
        throw new Error('Unhandled special key: ' + key);
    }
  }
} // I'm using the "alternate screenbuffer" of the terminal; this ensures the user's command-line
// history isn't cluttered with repeated interactive output from this demo. It has some annoying
// side-effects, which I try to mitigate below ...


function setupAltMode() {
  // Setting up the terminal, swtiching to the alternate screenbuffer;
  term.fullscreen(true);
  term.grabInput(true); // Horrible hack to freeze the Node.js event-loop long enough for a user to see output;

  function sleep(seconds) {
    var msecs = seconds * 1000;
    Atomics.wait(new Int32Array(new SharedArrayBuffer(4)), 0, 0, msecs);
  } // Ensuring a clean exit, and a return to the users' expected terminal-state


  function onExit(code) {
    term.grabInput(false);
    sleep(2);
    term.fullscreen(false);
  }

  process.once('exit', onExit);
  process.once('SIGINT', onExit);
}

function moveToEnd(buf) {
  var _buf$getContentSize = buf.getContentSize(),
      width = _buf$getContentSize.width,
      height = _buf$getContentSize.height;

  buf.moveToLine(height - 1);
  buf.moveToEndOfLine();
} // ... now we start actually setting up the terminal!


setupAltMode(); // Create a text-buffer to hold the user's input

var screen = new _terminalKit["default"].ScreenBuffer({
  dst: term
});
var notice = new _terminalKit["default"].ScreenBuffer({
  dst: screen,
  y: term.height - 1,
  x: term.width
});
var drawTimeout = null;
var drawPending = false;

function draw(buf) {
  var immediate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  // First, go ahead and aggressively perform any virtual drawing to parent buffers:
  var next = buf;

  do {
    next.draw();
    next = next.dst;
  } while (next.dst instanceof _terminalKit["default"].ScreenBuffer); // If there's been a draw recently ...


  if (immediate === false && null !== drawTimeout) drawPending = true; // ... else draw to the actual terminal, and schedule a check for the next draw.
  else {
      if (null !== drawTimeout) clearTimeout(drawTimeout);
      if (drawPending) drawPending = false;
      next.draw({
        delta: true
      });
      next.drawCursor();
      drawTimeout = setTimeout(function checkForScheduledDraw() {
        drawTimeout = null;

        if (drawPending) {
          next.draw({
            delta: true
          });
          next.drawCursor();
          drawPending = false;
        }
      }, 500);
    }
}

var noticeTimeout = null;

function showNotice(str) {
  var drawNow = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  if (noticeTimeout !== null) clearTimeout(noticeTimeout); // FIXME: This shouldn't be necessary; the below `put` at 0,0 should clear it, right? ...

  notice.fill({
    "char": ' '
  });
  notice.draw();
  notice.x = term.width - _terminalKit["default"].stringWidth(str) - 1;
  notice.put({
    x: 0,
    y: 0,
    markup: true
  }, str);
  drawNow ? draw(notice) : notice.draw();
  noticeTimeout = setTimeout(function clearNotice() {
    notice.fill({
      "char": ' '
    });
    draw(notice);
    noticeTimeout = null;
  }, 1000);
}

var intro = new _terminalKit["default"].ScreenBuffer({
  dst: screen,
  y: term.height - 2
});
intro.put({}, "Welcome! Edit the input above to see how it parses.\n(\u21A9  to watch the detailed, iterative process; \u2303 C to exit.)");
intro.draw();
var input = new _terminalKit["default"].TextBuffer({
  dst: screen,
  height: 3,
  x: 3,
  y: term.height - 5
});
input.setText(process.argv[2] || '');
moveToEnd(input);
input.drawCursor();
draw(input);
var output = new _terminalKit["default"].ScreenBuffer({
  dst: screen,
  y: 0,
  height: term.height - 6
});
term.on('key', handleKeypress.bind(null, input)); // ### Aaaaand we're off!

term.on('key', onChange.bind(null, input, output));